# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for collective.contentrules.mail doctest
"""
__docformat__ = 'restructuredtext'

from DateTime import DateTime


def addMember(self, username, fullname="", email="", roles=('Member', ), last_login_time=None):
    """Create an new member

    The password is always 'secret'
    """
    self.portal.portal_membership.addMember(username, 'secret', roles, [])
    member = self.portal.portal_membership.getMemberById(username)
    member.setMemberProperties({'fullname': fullname, 'email': email,
                                'last_login_time': DateTime(last_login_time)})


def setUpDefaultMembers(self):
    """Setup default members"""

    addMember(self, 'manager1', 'Manager one', roles=('Manager', 'Member'))
    addMember(self, 'manager2', 'Manager two', roles=('Manager', 'Member'))
    addMember(self, 'reviewer1', 'Reviewer one', roles=('Reviewer', 'Member'))
    addMember(self, 'reviewer2', 'Reviewer two', roles=('Reviewer', 'Member'))
    addMember(self, 'member1', 'Member one', roles=('Member', ))
    addMember(self, 'member2', 'Member two', roles=('Member', ))
    addMember(self, 'authenticated1', 'Authenticated one', roles=('Anonymous', ))
    addMember(self, 'anonymous1', 'Anonymous one', roles=('Anonymous', ))



from transaction import commit
import time

from zope.component import getUtility
from zope.component import getMultiAdapter

from plone.contentrules.engine.interfaces import IRuleAssignmentManager
from plone.contentrules.engine.interfaces import IRuleStorage

from collective.contentrules.mail.tests.base import TestCase

from Products.Five import zcml
from Products.Five import fiveconfigure

from Products.GenericSetup.interfaces import IBody
from Products.GenericSetup.context import TarballExportContext

from Products.PloneTestCase.layer import PloneSite
from Testing import ZopeTestCase

from zope.lifecycleevent.interfaces import IObjectModifiedEvent


class TestGenericSetup(TestCase):

    def afterSetUp(self):
        self.storage = getUtility(IRuleStorage)

    def testRuleInstalled(self):
        self.failUnless('test1' in self.storage)

    def testRulesConfigured(self):
        rule1 = self.storage['test1']
        self.assertEquals("Test rule 1", rule1.title)
        self.assertEquals("A test rule", rule1.description)
        self.assertEquals(IObjectModifiedEvent, rule1.event)
        self.assertEquals(True, rule1.enabled)
        self.assertEquals(False, rule1.stop)

        self.assertEquals(2, len(rule1.conditions))
        self.assertEquals("plone.conditions.PortalType", rule1.conditions[0].element)
        self.assertEquals(["Document", "News Item"], list(rule1.conditions[0].check_types))
        self.assertEquals("plone.conditions.Role", rule1.conditions[1].element)
        self.assertEquals(["Manager"], list(rule1.conditions[1].role_names))

        self.assertEquals(1, len(rule1.actions))
        self.assertEquals("collective.contentrules.mail.actions.Mail", rule1.actions[0].element)
        self.assertEquals("collective.contentrules.mail.model.base", rule1.actions[0].model)
        self.assertEquals("html", rule1.actions[0].mimetype)
        self.assertEquals("${default_from_email}", rule1.actions[0].source)
        self.assertEquals("${owner_emails}", rule1.actions[0].recipients)
        self.assertEquals(u"Your content was modified", rule1.actions[0].subject)
        self.assertEquals(u"Your content ${title} was modified.", rule1.actions[0].message)

    def testRuleAssigned(self):
        assignable = IRuleAssignmentManager(self.portal.news)
        self.assertEquals(1, len(assignable))

        self.assertEquals(True, assignable['test1'].enabled)
        self.assertEquals(False, assignable['test1'].bubbles)

    def testAssignmentOrdering(self):
        assignable = IRuleAssignmentManager(self.portal.news)
        self.assertEquals([u'test1'], assignable.keys())

    def testImportTwice(self):
        portal_setup = self.portal.portal_setup
        time.sleep(1)  # avoid timestamp colission
        portal_setup.runAllImportStepsFromProfile('profile-collective.contentrules.mail:testing')

        # We should get the same results as before
        self.testRuleInstalled()
        self.testRulesConfigured()
        self.testRuleAssigned()

    def testExport(self):
        site = self.portal
        context = TarballExportContext(self.portal.portal_setup)
        exporter = getMultiAdapter((site, context), IBody, name=u'plone.contentrules')

        expected = """\
<?xml version="1.0"?>
<contentrules>
 <rule name="test1" title="Test rule 1" description="A test rule"
    enabled="True" event="zope.lifecycleevent.interfaces.IObjectModifiedEvent"
    stop-after="False">
  <conditions>
   <condition type="plone.conditions.PortalType">
    <property name="check_types">
     <element>Document</element>
     <element>News Item</element>
    </property>
   </condition>
   <condition type="plone.conditions.Role">
    <property name="role_names">
     <element>Manager</element>
    </property>
   </condition>
  </conditions>
  <actions>
   <action type="collective.contentrules.mail.actions.Mail">
    <property name="mimetype">html</property>
    <property name="recipients">${owner_emails}</property>
    <property name="cc"></property>
    <property name="bcc">${contributor_emails}</property>
    <property name="source">${default_from_email}</property>
    <property name="message">Your content ${title} was modified.</property>
    <property name="model">collective.contentrules.mail.model.base</property>
    <property name="subject">Your content was modified</property>
   </action>
  </actions>
 </rule>
 <assignment name="test1" bubbles="False" enabled="True" location="/news"/>
</contentrules>
"""

        body = exporter.body
        self.assertEquals(expected.strip(), body.strip(), body)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestGenericSetup))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from zope.component import getUtility
from zope.app.form.browser.itemswidgets import DropdownWidget
from zope.component.interfaces import ComponentLookupError

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from kss.core import KSSView

from collective.contentrules.mail.interfaces import IMailModel


class ModelKSSView(KSSView):
    help_template = ViewPageTemplateFile("templates/model.pt")

    def updateModelDescription(self, model_id=''):
        # Get model and display help
        try:
            model = getUtility(IMailModel, model_id)
        except ComponentLookupError:
            model = None

        help = ""

        if model is not None:
            help = self.help_template(fields=model.fields)

        core = self.getCommandSet('core')
        core.replaceInnerHTML('#mail-model .modelDescription', help)
        return self.render()


class ModelWidget(DropdownWidget):

    help_template = ViewPageTemplateFile("templates/model.pt")

    def __init__(self, field, request):
        """Initialize the widget."""
        super(ModelWidget, self).__init__(field,
            field.vocabulary, request)

    def __call__(self):
        """See IBrowserWidget."""
        value = self._getFormValue()
        contents = []
        contents.append(self._div('value', self.renderValue(value)))
        contents.append(self._emptyMarker())

        # Get model and display model description
        # If no model selected, get the first value of vocabulary
        help = ""

        if not value and len(self.vocabulary):
            value = self.vocabulary._terms[0].value

        # Get model
        if value:
            try:
                model = getUtility(IMailModel, value)
            except ComponentLookupError:
                model = None

            if model is not None:
                help = self.help_template(fields=model.fields)

        contents.append(self._div("modelDescription", help))
        return self._div(self.cssClass, "\n".join(contents), id="mail-model")



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from zope.interface import Interface
from zope.schema import TextLine, Object, List, Tuple

from collective.contentrules.mail import MessageFactory as _


class IMailModel(Interface):
    """A named utility providing a mail model.

    The mail content rule will allow the user ot pick a mail model based on
    a vocabulary of all named utilities providing this interface. When the
    content rule action is executed, the object being acted upon will be
    adapted to the interface specified under `replacer_interface`.
    Substitutions will then be made based on variables matching field names
    (e.g. "${foobar}" matches the `foobar` field). The substituted values
    are obtained from the adapter.
    """

    title = TextLine(title=_(u"A friendly name for model", ))

    replacer_interface = Object(
        title=_(u"Mail replacer schema"),
        description=_(u"Interface providing word substitution in " \
                        "mail fields: source, recipients, subject, text"),
        schema=Interface)

    fields = List(
        title=_(u"Fields help text"),
        description=_(u"Exposes the variables provided by the replacer"),
        value_type=Tuple(title=_(u"Pair of (key, help text,)"),
                         value_type=TextLine(title=_(u"Name or help text"))))


class IMailReplacer(Interface):
    """Interface providing variables which can be used in mail fields:
    source, recipients, subject, text. This is the default implementation,
    which should work on any CMF content providing the IDublinCore interface.

    It is possible to extend this with other attributes and provide a new
    IMailModel utility with a different interface provided as the
    `replacer_interface`.
    """

    id = TextLine(title=_(u"Id of content", ))

    title = TextLine(title=_(u"Title of content", ))

    description = TextLine(title=_(u"Description of content", ))

    url = TextLine(title=_(u"URL to access content", ))

    relative_url = TextLine(
        title=_(u"Relative URL from portal to access content", ))

    portal_url = TextLine(title=_(u"URL of portal", ))

    owner_id = TextLine(title=_(u"Login of content ower", ))

    owner_fullname = TextLine(title=_(u"Full name of content owner", ))

    owner_emails = TextLine(
        title=_(u"Emails of users having Owner role on content", ))

    reader_emails = TextLine(
            title=_(u"Emails of users having Reader role on content", ))

    contributor_emails = TextLine(
            title=_(u"Emails of users having Contributor role on content", ))

    editor_emails = TextLine(
            title=_(u"Emails of users having Editor role on content", ))

    reviewer_emails = TextLine(
            title=_(u"Emails of users having Reviewer role on content", ))

    default_from_email = TextLine(title=_(u"Email address of default sender", ))

    default_from_name = TextLine(title=_(u"Full name of default sender", ))

    review_state = TextLine(title=_(u"State of content", ))



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from Acquisition import aq_inner
from OFS.SimpleItem import SimpleItem
from zope.component import adapts
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.interface import Interface
from zope.interface import implements
from zope.formlib import form
from zope.schema import TextLine
from zope.schema import Text
from zope.schema import Choice
from zope.schema import getFieldNamesInOrder

from Products.CMFCore.utils import getToolByName

from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IRuleElementData
from plone.contentrules.rule.interfaces import IExecutable

from collective.contentrules.mail import MessageFactory as _
from collective.contentrules.mail import LOG
from collective.contentrules.mail.browser.widget import ModelWidget
from collective.contentrules.mail.interfaces import IMailModel

from smtplib import SMTPException
from Products.MailHost.MailHost import MailHostError


class IMailAction(Interface):
    """Definition of the configuration available for a mail action
    """

    model = Choice(
        title=_(u"Mail model"),
        required=True,
        vocabulary="collective.contentrules.mail.vocabulary.model", )

    mimetype = Choice(
        title=_(u"Mail mimetype"),
        required=True,
        vocabulary="collective.contentrules.mail.vocabulary.mimetype", )

    subject = TextLine(
        title=_(u"Subject"),
        description=_(u"Subject of the message"),
        required=True)

    source = TextLine(
        title=_(u"Email source"),
        description=_("The email address that sends the email. If no email is" \
                      " provided here, it will use the portal from address."),
        required=False)

    recipients = TextLine(
        title=_(u"Email recipients"),
        description=_("The email where you want to send this message. To send" \
            " it to different email addresses, just separate them with commas."),
        required=True)

    cc = TextLine(
        title=_(u"CC recipients"),
        description=_("The email to receive a copy of this message. To send" \
            " it to different email addresses, just separate them with commas."),
        required=False)

    bcc = TextLine(
        title=_(u"BCC recipients"),
        description=_("The email to receive a blind copy of this message. To" \
     " send it to different email addresses, just separate them with commas."),
        required=False)

    message = Text(
        title=_(u"Message"),
        description=_(u"Type in here the message that you want to mail."),
        required=True)


class MailAction(SimpleItem):
    """
    The implementation of the action defined before
    """
    implements(IMailAction, IRuleElementData)

    model = ''
    mimetype = ''
    subject = ''
    source = ''
    recipients = ''
    cc = ''
    bcc = ''
    message = ''

    element = 'collective.contentrules.mail.actions.Mail'

    @property
    def summary(self):
        return _(u"Email report to ${recipients}",
                 mapping=dict(recipients=self.recipients))


class MailActionExecutor(object):
    """The executor for this action.
    """
    implements(IExecutable)
    adapts(Interface, IMailAction, Interface)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        # Get event objet
        obj = self.event.object

        utool = getToolByName(aq_inner(self.context), "portal_url")
        portal = utool.getPortalObject()

        site_charset = 'utf-8'
        email_charset = portal.getProperty('email_charset')

        # Get replacer interface from model
        model = getUtility(IMailModel, self.element.model)
        replacer_interface = model.replacer_interface

        # Extract all variables from this replacer
        replacer = replacer_interface(obj, None)
        if replacer is None:
            LOG.debug(u"Could not send email. The replacer is not applicable for this type of object.")
            return False

        words = {}
        for word_id in getFieldNamesInOrder(replacer_interface):
            replacement_value = getattr(replacer, word_id)
            if isinstance(replacement_value, str):
                replacement_value = replacement_value.decode(site_charset)
            words[word_id] = replacement_value

        # Apply word substitution on every mail fields
        def substitute(text):
            if not text:
                return ""

            for word_id, replacement_value in words.items():
                text = text.replace(u"${%s}" % word_id, replacement_value)

            return text.encode(site_charset)

        source = self.element.source
        if source:
            source = substitute(self.element.source)
        recipients = substitute(self.element.recipients)
        cc = substitute(self.element.cc)
        bcc = substitute(self.element.bcc)
        subject = substitute(self.element.subject)
        message = substitute(self.element.message)

        def processRecipients(recipients=''):
            address_list = []
            for email in recipients.split(','):
                email = email.strip()
                if not email:
                    # Remove empty address
                    continue
                address_list.append(email)
            return address_list

        # Process recipients
        recipient_list = processRecipients(recipients)
        cc_list = [e for e in processRecipients(cc) if e not in recipients]
        bcc_list = [e for e in processRecipients(bcc) if e not in recipients]

        if not recipient_list:
            # Because there are no recipients, do not send email
            LOG.info(u"""Do not send email "%s": no recipients defined.""" %
                     model.title)
            return False

        recipients = ",".join(recipient_list)
        cc = ",".join(cc_list)
        bcc = ",".join(bcc_list)

        # Process source

        if not source:
            # no source provided, looking for the site wide from email
            # address
            from_address = portal.getProperty('email_from_address')

            if not from_address:
                raise ValueError, "You must provide a source address for this\
action or enter an email in the portal properties"

            from_name = portal.getProperty('email_from_name')
            source = "%s <%s>" % (from_name, from_address)

        # Encode text using mail charset

        if email_charset != site_charset:
            source = source.decode(site_charset).encode(email_charset)
            recipients = recipients.decode(site_charset).encode(email_charset)
            cc = cc.decode(site_charset).encode(email_charset)
            bcc = bcc.decode(site_charset).encode(email_charset)
            subject = subject.decode(site_charset).encode(email_charset)
            message = message.decode(site_charset).encode(email_charset)

        # Send email
        mailhost = getToolByName(aq_inner(self.context), "MailHost")

        if not mailhost:
            raise ComponentLookupError, \
                "You must have a Mailhost utility to execute this action"

        mimetype = self.element.mimetype

        try:
            mailhost.secureSend(message, recipients, source,
                                subject=subject, subtype=mimetype,
                                mcc=cc, mbcc=bcc,
                                charset=email_charset, debug=False)
        except (MailHostError, SMTPException, ), e:
            LOG.exception(u"Failed to send mail")
            return False

        return True


class MailAddForm(AddForm):
    """
    An add form for the mail action
    """
    form_fields = form.FormFields(IMailAction)
    form_fields['model'].custom_widget = ModelWidget
    label = _(u"Add Mail Action")
    description = _(u"A mail action can mail different recipient.")
    form_name = _(u"Configure element")

    def create(self, data):
        a = MailAction()
        form.applyChanges(a, self.form_fields, data)
        return a


class MailEditForm(EditForm):
    """
    An edit form for the mail action
    """
    form_fields = form.FormFields(IMailAction)
    form_fields['model'].custom_widget = ModelWidget
    label = _(u"Edit Mail Action")
    description = _(u"A mail action can mail different recipient.")
    form_name = _(u"Configure element")



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for collective.contentrules.mail doc strings
"""
__docformat__ = 'restructuredtext'

import unittest
import doctest
import sys
import os

current_dir = os.path.abspath(os.path.dirname(__file__))


def doc_suite(test_dir, globs=None):
    """Returns a test suite, based on doc tests strings found in /*.py"""
    suite = []
    if globs is None:
        globs = globals()

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE)

    package_dir = os.path.split(test_dir)[0]
    if package_dir not in sys.path:
        sys.path.append(package_dir)

    # filtering files on extension
    docs = [doc for doc in
            os.listdir(package_dir) if doc.endswith('.py')]
    docs = [doc for doc in docs if not doc.startswith('__')]

    for test in docs:
        fd = open(os.path.join(package_dir, test))
        content = fd.read()
        fd.close()
        if '>>> ' not in content:
            continue
        test = test.replace('.py', '')
        suite.append(doctest.DocTestSuite('collective.contentrules.mail.%s' % test,
                                          optionflags=flags,
                                          globs=globs))

    # filtering files on extension
    docs = [doc for doc in
            os.listdir(os.path.join(package_dir, 'browser')) if doc.endswith('.py')]
    docs = [doc for doc in docs if not doc.startswith('__')]

    for test in docs:
        fd = open(os.path.join(package_dir, 'browser', test))
        content = fd.read()
        fd.close()
        if '>>> ' not in content:
            continue
        test = test.replace('.py', '')
        suite.append(doctest.DocTestSuite('collective.contentrules.mail.browser.%s' % test,
                                          optionflags=flags,
                                          globs=globs))

    return unittest.TestSuite(suite)


def test_suite():
    """returns the test suite"""
    return doc_suite(current_dir)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# Initialize logger
import logging
from zope.i18nmessageid import MessageFactory as BaseMessageFactory

PROJECTNAME = 'collective.contentrules.mail'
LOG = logging.getLogger(PROJECTNAME)
MessageFactory = BaseMessageFactory(PROJECTNAME)



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from collective.contentrules.mail.interfaces import IMailModel
from collective.contentrules.mail import MessageFactory as _
from zope.component import getUtilitiesFor
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


class ModelVocabulary(object):
    """Vocabulary factory listing all mail models
    """

    implements(IVocabularyFactory)

    def __call__(self, context):
        terms = [SimpleTerm(name, title=utility.title)
                 for name, utility in getUtilitiesFor(IMailModel)]
        return SimpleVocabulary(terms)


ModelVocabularyFactory = ModelVocabulary()


class MimetypeVocabulary(object):
    """Vocabulary factory listing mail mimetypes
    """

    implements(IVocabularyFactory)

    def __call__(self, context):
        terms = (
            SimpleTerm(
                u"plain",
                title=_(u"text/plain", )),
            SimpleTerm(
                u"html",
                title=_(u"text/html", )),
            )

        return SimpleVocabulary(terms)


MimetypeVocabularyFactory = MimetypeVocabulary()



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""" defines a test class and its Plone Site layer for plone tests
"""
from plone.app.testing.bbb import PloneTestCaseFixture, PloneTestCase
from plone.app import testing
from plone.app.testing import setRoles, TEST_USER_ID
from plone.testing import z2

# fake mailhost
from Products.MailHost import MailHost


class TestMailHost(MailHost.MailHost):

    def _send(self, mfrom, mto, messageText):
        """Fake sender"""
        print messageText


class CCMFixture(PloneTestCaseFixture):

    def setUpZope(self, app, configurationContext):
        super(PloneTestCaseFixture, self).setUpZope(app, configurationContext)
        import collective.contentrules.mail
        self.loadZCML(package=collective.contentrules.mail)
        z2.installProduct(app, 'collective.contentrules.mail')

    def setUpPloneSite(self, portal):
        super(PloneTestCaseFixture, self).setUpPloneSite(portal)

        # prepare structure
        if 'news' not in portal:
            setRoles(portal, TEST_USER_ID, ['Manager'])
            portal.invokeFactory('Folder', 'news', title='News Folder')
            setRoles(portal, TEST_USER_ID, ['Member'])

        # install product
        testing.applyProfile(portal, 'collective.contentrules.mail:default')

        # install testing profile
        from Products.GenericSetup import EXTENSION, profile_registry
        profile_registry.registerProfile('testing',
            "collective.contentrules.mail testing",
            'Used for testing only',
            'profiles/testing',
            'collective.contentrules.mail',
            EXTENSION)
        testing.applyProfile(portal, 'collective.contentrules.mail:testing')
        # patch mail host
        #portal._old = MailHost.MailHost
        #MailHost.MailHost = TestMailHost

        portal.manage_changeProperties(email_from_name='Site Administrator')

    def tearDownZope(self, app):
        super(PloneTestCaseFixture, self).tearDownZope(app)
        z2.uninstallProduct(app, 'collective.contentrules.mail')

PTC_FIXTURE = CCMFixture()
PTC_FUNCTIONAL_TESTING = testing.FunctionalTesting(
    bases=(PTC_FIXTURE, ), name='PloneTestCase:Functional')


class TestCase(PloneTestCase):
    """ Base class used for test cases """

    layer = PTC_FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from zope.interface import implements
from zope.schema import getFieldsInOrder

from collective.contentrules.mail.interfaces import IMailModel
from collective.contentrules.mail.interfaces import IMailReplacer
from collective.contentrules.mail import MessageFactory as _


class MailModel(object):
    """A mail model described by interface.

    Create instances of this (as shown for DefaultMailModel below) and
    register these as unique named utilities providing IMailModel.
    """

    implements(IMailModel)

    def __init__(self, title, replacer_interface):
        self.title = title
        self.replacer_interface = replacer_interface

    @property
    def fields(self):
        fields = []

        # List of variables provided by replacer interface
        for name, field in getFieldsInOrder(self.replacer_interface):
            fields.append((name, field.title))

        return fields

# The default mail model, which uses the default mail replacer.
DefaultMailModel = MailModel(title=_(u"Standard model for plone content"),
                             replacer_interface=IMailReplacer)



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from zope.interface import implements
from zope.component import adapts

from Products.CMFCore.WorkflowTool import WorkflowException
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IDublinCore
from collective.contentrules.mail.interfaces import IMailReplacer


class MailReplacer(object):
    """Provides attributes which can be used in a mail model"""

    implements(IMailReplacer)
    adapts(IDublinCore)

    def __init__(self, context):
        self.context = context
        self.utool = getToolByName(context, "portal_url")
        self.portal = self.utool.getPortalObject()
        self.mtool = getToolByName(context, "portal_membership")
        self.gtool = getToolByName(context, "portal_groups")
        self.wtool = getToolByName(context, "portal_workflow")
        self.acl_users = getToolByName(context, "acl_users")

    def _getRoleEmails(self, roles):
        # Returns a list of emails for users having the specified roles
        # @param roles: Set of roles
        roles = set(roles)
        local_roles = self.acl_users._getAllLocalRoles(self.context)
        emails = []

        for role in roles:
            members = self.acl_users.portal_role_manager.listAssignedPrincipals(role)
            for member_id in members:
                emails += self._getPrincipalEmail(member_id)

        for item_id, item_roles in local_roles.items():
            # Check if local roles has those that we wanted to extract
            if not roles.intersection(item_roles):
                continue

            emails += self._getPrincipalEmail(item_id)

        return set(emails)

    def _getPrincipalEmail(self, principal_id):
        emails = set()
        members = set()
        member = self.mtool.getMemberById(principal_id)

        if member is not None:
            members.add(member)
        else:
            # Member does not exist. Check if it is a group
            group = self.gtool.getGroupById(principal_id)

            if group is not None:
                members = members.union(group.getGroupMembers())

        # Get all emails from fetched members
        for member in members:
            email = member.getProperty('email', None)
            if not email:
                continue

            emails.add(email)

        return emails

    @property
    def owner_emails(self):
        return ", ".join(self._getRoleEmails(['Owner']))

    @property
    def reader_emails(self):
        return ", ".join(self._getRoleEmails(['Reader']))

    @property
    def contributor_emails(self):
        return ", ".join(self._getRoleEmails(['Contributor']))

    @property
    def editor_emails(self):
        return ", ".join(self._getRoleEmails(['Editor']))

    @property
    def reviewer_emails(self):
        return ", ".join(self._getRoleEmails(['Reviewer']))

    @property
    def review_state(self):
        try:
            return self.wtool.getInfoFor(self.context, "review_state")
        except WorkflowException:
            return ''

    @property
    def id(self):
        return self.context.getId()

    @property
    def title(self):
        return self.context.title_or_id()

    @property
    def description(self):
        return self.context.Description()

    @property
    def url(self):
        return self.context.absolute_url()

    @property
    def relative_url(self):
        return self.utool.getRelativeUrl(self.context)

    @property
    def portal_url(self):
        return self.utool()

    @property
    def owner_id(self):
        return self.context.Creator()

    @property
    def owner_fullname(self):
        owner = self.mtool.getMemberById(self.owner_id)

        if owner is None:
            # Owner is not registered in portal_metadata, so fullname can't
            # be retrieved
            return self.owner_id

        # Returns fullname
        return owner.getProperty('fullname')

    @property
    def default_from_email(self):
        return self.portal.getProperty('email_from_address')

    @property
    def default_from_name(self):
        return self.portal.getProperty('email_from_name', 'Administrator')



from setuptools import setup, find_packages

version = '0.7'

setup(name='collective.contentrules.mail',
      version=version,
      description="Flexible mail content rule",
      long_description=(open("README.rst").read() + "\n" +
                        open("CHANGES.rst").read()),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Framework :: Plone",
          "Framework :: Plone :: 4.0",
          "Framework :: Plone :: 4.1",
          "Framework :: Plone :: 4.2",
          "Framework :: Plone :: 4.3",
          "Framework :: Plone :: 5.0",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Topic :: Software Development :: Libraries :: Python Modules",
          ],
      keywords='',
      author='Ingeniweb',
      author_email='support@ingeniweb.com',
      url='https://github.com/collective/collective.contentrules.mail',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective', 'collective.contentrules'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'plone.app.kss',
          'zope.app.form',

      ],
      extras_require = {
        "test": [
            "plone.app.testing>=4.2.3",
            "Products.PloneTestCase"],
      },
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
# Copyright (c) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for collective.contentrules.mail doctest
"""
__docformat__ = 'restructuredtext'

import unittest
import doctest
import sys
import os

from Testing.ZopeTestCase import FunctionalDocFileSuite
from Products.Five.testbrowser import Browser

from base import TestCase

current_dir = os.path.dirname(__file__)


def doc_suite(test_dir, setUp=None, tearDown=None, globs=None):
    """Returns a test suite, based on doctests found in /doctest."""
    suite = []
    if globs is None:
        globs = globals()

    # preparing a few elements
    globs['test_dir'] = current_dir
    browser = Browser()
    browser.handleErrors = False
    globs['browser'] = browser

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE |
             doctest.REPORT_UDIFF)

    package_dir = os.path.split(test_dir)[0]
    if package_dir not in sys.path:
        sys.path.append(package_dir)

    docs = []
    for dir_ in ('doctests', 'docs'):
        doctest_dir = os.path.join(package_dir, dir_)

        # filtering files on extension
        docs.extend([os.path.join(doctest_dir, doc) for doc in
                     os.listdir(doctest_dir) if doc.endswith('.txt')])

    for test in docs:
        suite.append(FunctionalDocFileSuite(test, optionflags=flags,
                                            globs=globs, setUp=setUp,
                                            tearDown=tearDown,
                                            test_class=TestCase,
                                            module_relative=False))

    return unittest.TestSuite(suite)


def test_suite():
    """returns the test suite"""
    return doc_suite(current_dir)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
